local encounters = HudMap:GetModule("Encounters")
local HudMap = _G.HudMap
local UnitName, UnitIsDead = _G.UnitName, _G.UnitIsDead

local marrowgar = {
	spikes = {},
	spikesOn = {},
	registerOn = {"SPELL_SUMMON", "UNIT_DIED"},
	SPELL_SUMMON = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		if spellID == 69062 or spellID == 72669 or spellID == 72670 then
			self.spikes[destGUID] = HudMap:PlaceRangeMarkerOnPartyMember("timer", destName, 2, 8, 1, 0, 0, 0.6)
			self.spikesOn[sourceGUID] = destGUID
		end
	end,
	UNIT_DIED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		local marker = self.spikes[destGUID]
		if not marker then 
			local pguid = self.spikesOn[sourceGUID]
			if pguid then
				marker = self.spikes[pguid]
			end
		end
		if marker then marker:Free() end
	end
}

local festergut = {
	registerOn = {"SPELL_AURA_APPLIED"},
	SPELL_AURA_APPLIED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		if spellID == 69279 then
			HudMap:PlaceRangeMarkerOnPartyMember("timer", destName, 8, 12, 1, 0.5, 0, 1)
		end
	end
}

local rotface = {
	registerOn = {"SPELL_CAST_START", "SPELL_AURA_APPLIED"},
	showSplashes = function()
		for index, unit in encounters.group() do
			if not UnitIsDead(unit) then
				local x, y = HudMap:GetUnitPosition(u)
				HudMap:PlaceRangeMarker("timer", x, y, 6, 10, 0, 1, 0, 1)
			end
		end
	end,
	SPELL_CAST_START = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		if spellID == 69839 then
			encounters:Delay(self.showSplashes, 4)
		end
	end,
	SPELL_AURA_APPLIED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		if spellID == 72272 or spellID == 72273 then
			HudMap:PlaceRangeMarkerOnPartyMember("timer", destName, 8, 6, 1, 0.5, 0, 1)
		end
	end
}

local putricide = {
	registerOn = {"SPELL_CAST_SUCCESS", "SPELL_AURA_APPLIED", "SPELL_AURA_REMOVED", "UNIT_DIED"},
	plagueOn = {},
	gooTargetFunc = function()
		local gooTarget = encounters:GetMobTarget(36678)
		if gooTarget then
			local x, y = HudMap:GetUnitPosition(gooTarget)				
			HudMap:PlaceRangeMarker("timer", x, y, 7, 7, 0, 1, 0, 1)
		end
	end,
	SPELL_CAST_SUCCESS = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- Malleable Goo. Find Putricide's target and mark their splash zone. Can we mark bounce spots as well?
		if spellID == 72295 or spellID == 74280 or spellID == 72615 or spellID == 74281 then
			encounters:Delay(self.gooTargetFunc, 0.1)
		end
	end,
	SPELL_AURA_APPLIED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- Player picks up Unbound Plague
		if spellID == 72855 or spellID == 72856 then
			self.plagueOn[destGUID] = HudMap:PlaceRangeMarkerOnPartyMember("timer", destName, 3, 60, 1, 0.5, 0, 1)
		end
	end,
	SPELL_AURA_REMOVED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- Player loses Unbound Plague
		if spellID == 72855 or spellID == 72856 then
			local marker = self.plagueOn[destGUID]
			if marker then
				marker:Free()
			end
		end
	end,
	UNIT_DIED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- Player with Plague dies
		local marker = self.plagueOn[destGUID]
		if marker then
			marker:Free()
		end
	end
}

local princes = {
	registerOn = {"SPELL_CAST_START"},
	shockTargetFunc = function()
		local shockTarget = encounters:GetMobTarget(37970)
		if shockTarget then
			local x, y = HudMap:GetUnitPosition(shockTarget)				
			HudMap:PlaceRangeMarker("timer", x, y, 13, 45, 1, 1, 1, 1):Appear():Rotate(360, 15)
		end
	end,
	SPELL_CAST_START = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- AOE shock vortex
		if spellID == 72037 then
			encounters:Delay(self.shockTargetFunc, 0.1)
		-- Empowered shock vortex
		elseif spellID == 72039 or spellID == 73037 or spellID == 73038 or spellID == 73039 then
			encounters:RangeCheckAll("timer", 12, 5, 1, 1, 1, 0.4, 1, 0, 0, 0.4, true)
		end
	end
}

local bloodqueen = {
	registerOn = {"SPELL_CAST_START"},
	SPELL_CAST_START = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- Fear
		if spellID == 73070 then
			for i = 1, 25 do
				local u = "raid" .. i
				local n = UnitName(u)
				if n and not UnitIsDead(u) and not UnitIsUnit("player", u) then
					HudMap:PlaceRangeMarkerOnPartyMember("timer", u, 6, 10, 0.4, 0.4, 1, 1):Rotate(360, 10)
				end
			end
		end
	end
}

local sindragosa = {
	registerOn = {"SPELL_AURA_APPLIED", "SPELL_AURA_REMOVED"},
	timers = {},
	SPELL_AURA_APPLIED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- Frost beacon
		if spellID == 70126 then
			HudMap:PlaceRangeMarkerOnPartyMember("timer", destName, 10, 7, 0.4, 0.4, 1, 1):Rotate(360, 7)
		elseif spellID == 69766 and encounters:IsHeroic() then
			if self.timers[destName] then
				self.timers[destName]:Free()
			end
			self.timers[destName] = HudMap:PlaceRangeMarkerOnPartyMember("highlight", destName, 20, nil, 1, 0, 0, 0.5):Appear()
		end
	end,
	SPELL_AURA_REMOVED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		if spellID == 69766 and self.timers[destName] then
			self.timers[destName]:Free()
		end
	end
}

encounters:RegisterModule(marrowgar)
encounters:RegisterModule(festergut)
encounters:RegisterModule(rotface)
encounters:RegisterModule(putricide)
encounters:RegisterModule(princes)
encounters:RegisterModule(bloodqueen)
encounters:RegisterModule(sindragosa)